#
# $Copyright: Copyright (c) 2011 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $RCSfile: hadrlib.pm,v $ $Revision: 1.9.2.8 $
#

package hadrlib;
use ArrayRep;
@ISA = ("ArrayRep");

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %default);
use ag_i18n_inc;
use Sys::Hostname;

@EXPORT = qw(
);
$VERSION = '1.00';

require 5.000;

use lib qw(.);
use hadrconfig() ;
use vars qw(%CONF);
*CONF = \%hadrconfig::CONF;
my $SU = $CONF{SU};
my $PASSWDFILE = $CONF{PASSWDFILE};

my $lfpath = './.';
my $lfext = "_LCK.dont_remove";

%default = (
 	resname => '',
 
 	VADCtlMode => '',
 
 	# Elements to store the resource atributes
 	# ========================================
 
 	# Db2dbRes is a reference to the resource of type DB2 which
 	# manages the replicated instance
 	Db2dbRes => '',
 
 	# DBName - the name of the replicated database - one DB2 server
 	# can serve multiple databases
 	DBName => '',
 
 	# HADROper - the operational mode of the Db2HADR resoource
 	# values: LINK or SWITCH
 	HADROper => '',
 
 	# UseForce - option to force the open of a primary database
 	# without connection to the standby
 	UseForce => 0,

	# PeerWindowOnly - option to force to peer window only option
	# while takeover by force.
	PeerWindowOnly => 0, 
 
 	# Encoding - support for non C OS environments
 	ENCODE => ''
);

sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;

	my $ret;
	my $self = { %default };

	$self->{'resname'} = shift;

	my @attrs = @_;

	bless($self, $class);

	# Check for VAD controlmode return value if > 0
	my $clus_prod = $ENV{"CLUSTER_PRODUCT"};
	$clus_prod = "VCS" if (! defined $clus_prod);
	$self->{'VADCtlMode'} = 0;
	if ($clus_prod eq "VAD") {
		($ret, $self->{'VADCtlMode'}) = VCSAG_GET_ATTR_VALUE ("ControlMode", -1, 1, @attrs);
		return undef if ($ret != $VCSAG_SUCCESS);
	}

	# Add some common variables
	$self->{'nodename'} = `$ag_i18n_inc::HASYS -nodeid 2>&1`;
	$self->{'nodename'} =~ s/^.*name:\s+(\S+)\s+.*/$1/;
	chomp $self->{'nodename'};

	# Get the attributes
	($ret, $self->{'Db2dbRes'}) = VCSAG_GET_ATTR_VALUE ("Db2dbRes", -1, 1, @attrs);
	if ($ret != $VCSAG_SUCCESS || $self->{'Db2dbRes'} eq "") {
		VCSAG_LOG_MSG("E", "Attribute Db2dbRes is not defined", 201);
		return undef;
	}
	my $pstat = `$ag_i18n_inc::HARES -state $self->{'Db2dbRes'} 2>&1`;
	if ($?) {
		VCSAG_LOG_MSG("E", "Resource $self->{'Db2dbRes'} does not exist in the local cluster, please adjust attribute Db2dbRes", 202, "$self->{'Db2dbRes'}");
		return undef;
	}

	my $iowner = `$ag_i18n_inc::HARES -value $self->{'Db2dbRes'} DB2InstOwner $self->{'nodename'} 2>&1`;
	if ($?) {
		VCSAG_LOG_MSG("E", "Can not get value for instance owner from resource $self->{'Db2dbRes'}", 206, "$self->{'Db2dbRes'}");
		return undef;
	}

	my $ihome = `$ag_i18n_inc::HARES -value $self->{'Db2dbRes'} DB2InstHome $self->{'nodename'} 2> /dev/null`;
	if ($?) {
		VCSAG_LOG_MSG("E", "Can not get value for instance home from resource $self->{'Db2dbRes'}", 207, "$self->{'Db2dbRes'}");
		return undef;
	}
	chomp($iowner, $ihome);

	($ret, $self->{'DBName'}) = VCSAG_GET_ATTR_VALUE ("DBName", -1, 1, @attrs);
	if ($ret != $VCSAG_SUCCESS || $self->{'DBName'} eq "") {
		VCSAG_LOG_MSG("E", "Database name is not defined", 203);
		return undef;
	}

	($ret, $self->{'HADROper'}) = VCSAG_GET_ATTR_VALUE ("HADROper", -1, 1, @attrs);
	if ($ret != $VCSAG_SUCCESS || $self->{'HADROper'} eq "") {
		VCSAG_LOG_MSG("E", "Attribute HADROper is not defined", 204);
		return undef;
	}
	unless ($self->{'HADROper'} =~ /(link|switch)/i) {
		VCSAG_LOG_MSG("E", "Attribute HADROper has to be LINK | SWITCH", 205);
	}
	($ret, $self->{'ENCODE'}) = VCSAG_GET_ATTR_VALUE ("Encoding", -1, 1, @attrs);
	if ($self->{'ENCODE'}) {
		$self->{'ENCODE'} = "-encoding $self->{'ENCODE'}";
	}
	
	($ret, $self->{'PeerWindowOnly'}) = VCSAG_GET_ATTR_VALUE ("PeerWindowOnly", -1, 1, @attrs);
	if ($ret != $VCSAG_SUCCESS || $self->{'PeerWindowOnly'} eq "") {
                VCSAG_LOG_MSG("E", "PeerWindowOnly is not defined", 206);
                return undef;
        }
	unless (($self->{'PeerWindowOnly'} == 0) or ($self->{'PeerWindowOnly'} == 1)) {
		VCSAG_LOG_MSG("E", "Attribute PeerWindowOnly has to be 0 or 1", 207);
	}
	
	#
	# Create db2 CLI sequence for this user
	my $UseCSH = 0;
	open (PASSWD, $PASSWDFILE);
	while (<PASSWD>) {
		next unless /^$iowner:/;
		$UseCSH = 1 if /csh$/;
		last;
	}
	while (<PASSWD>) {next;}	# empty pipe
	close PASSWD;

	if ($UseCSH) {
		$self->{'db2cmd'} = "$SU $iowner -c \'source $ihome/sqllib/db2cshrc; $ihome/sqllib/bin/db2 <DB2StmtToken>\'";
	} else {
		$self->{'db2cmd'} = "$SU $iowner -c \'. $ihome/sqllib/db2profile; $ihome/sqllib/bin/db2 <DB2StmtToken>\'";
	}
	if (-x "$ihome/sqllib/adm/db2pd") {
		$self->{'db2pd'} = "DB2INSTANCE=\"$iowner\" export DB2INSTANCE; $SU $iowner -c \'$ihome/sqllib/adm/db2pd -db $self->{'DBName'} -hadr\' |";
	}
	if (-x "$ihome/sqllib/bin/db2_kill") {
		$self->{'db2kill'} = "DB2INSTANCE=\"$iowner\" export DB2INSTANCE; $SU $iowner -c \'$ihome/sqllib/bin/db2_kill\'";
	}
	if (-x "$ihome/sqllib/adm/db2licm") {
		$self->{'db2licm'} = "$SU $iowner -c \'$ihome/sqllib/adm/db2licm -v 2>&1\'";
	}

	return $self;
}

sub DESTROY {}

################# FUNCTION DEFN START ##########################

# Function : exec_db2
# Desc     : Execute a single DB2 command and analyze output
# Returns  : 0 - Output did not start with a SQL or DB2 error code
#          : 1 - output started with SQL or DB2 error code 
#
sub exec_db2 {
	my $self = shift;
	my $db2stat = shift;
	my $db2exe = $self->{'db2cmd'};

	$db2exe =~ s/<DB2StmtToken>/$db2stat/;
	$self->{'db2out'} = `$db2exe`;
	VCSAG_LOGDBG_MSG(1, "$db2exe Command output :: $self->{'db2out'}", 0, "$db2exe", "$self->{'db2out'}");
	return 1 if $self->{'db2out'} =~ /^SQL/;
	return 0;
}

# Function : get_hadr_config
# Desc     : Retrieve HADR state information from database configuration
# Updates  : $db2out - complete block of HADR snapshot contents
#
sub get_hadr_config {
	my $self = shift;

	my $db2cmd = "GET DATABASE CONFIGURATION FOR $self->{'DBName'}";

	return 1 if $self->exec_db2($db2cmd);

	my @lines = split(/\n+/, $self->{'db2out'});
	$self->{'db2out'} = "";
	$self->{'rollfor'} = "";
	$self->{'conrole'} = "";
	for (@lines) {
		if (/Rollforward pending/) {
			(undef, $self->{'rollfor'}) = split /\s+=\s+/ if /Rollforward pending/;
			$self->{'db2out'} .= "$_\n";
		}
		next unless /^\s+HADR/;
		$self->{'db2out'} .= "$_\n";
		(undef, $self->{'conrole'}) = split /\s+=\s+/ if /HADR database role/;
		last if /\(HADR_SYNCMODE\)/;
	}
	return 0;
}

# Function : get_hadr_state
# Desc     : Retrieve HADR state information from database snapshot
# Updates  : $self->{'db2out'} - complete block of HADR snapshot contents
#
sub get_hadr_state {
	my $self = shift;

	my $db2cmd = "GET SNAPSHOT FOR DATABASE ON $self->{'DBName'}";
	my $found = 0;
	my $db_state_found = 0;

	return 1 if $self->exec_db2($db2cmd);

	my @lines = split(/\n+/, $self->{'db2out'});
	$self->{'db2out'} = "";
	for (@lines) {
		if ($db_state_found == 0) {
			next unless /^Database status/;
			$db_state_found = 1;
			(undef, $self->{'db_state'}) = split /\s+=\s+/;
		}
		next unless /^HADR Status/ || $found;
		$found = 1;
		$self->{'db2out'} .= "$_\n";
		(undef, $self->{'hrole'}) = split /\s+=\s+/ if /^\s+Role/;
		(undef, $self->{'hstat'}) = split /\s+=\s+/ if /^\s+State/;
		(undef, $self->{'cstat'}) = split /\s+=\s+/ if /^\s+Connection status/;
		(undef, $self->{'sync_mode'}) = split /\s+=\s+/ if /^\s+Synchronization mode/;
		last if /Log gap/;
	}
	return 0;
}

# Function : get_remote_info
# Desc     : Retrieve the name of the remote node and cluster
# Returns  : 0 - success 
#          : 1 - one of the fields is empty

sub get_remote_info {
	my $self = shift;
	my $gl_res;

	if ($self->{'HADROper'} =~ /SWITCH/i) {
		$gl_res = $self->{'resname'};
	} else {
		$gl_res = $self->{'peer'};
	}
	$self->{'gl_grp'} = `$ag_i18n_inc::HARES -value $gl_res Group 2>&1`;
	chomp $self->{'gl_grp'};

	open(RDSP, "$ag_i18n_inc::HAGRP -state $self->{'gl_grp'} 2>&1 |");
	while (<RDSP>) {
		next if /^#/;
		next if /localclus:\s/;
		my @line = split();
		($self->{'rclus'}, $self->{'rnode'}) = split(':', $line[2]);
		if ($self->{'rnode'} eq "" ) {
			#
			# This is a single cluster HADR implementation
			#
			$self->{'rnode'} = $self->{'rclus'};
			$self->{'rclus'} = "localclus";
		}

		last;
	}
	while (<RDSP>) {next;}	# empty pipe
	close RDSP;

	return 1 unless (defined $self->{'rclus'} && defined $self->{'rnode'});
	return 0;
}

# Function : get_peer_res_name
# Desc     : Retrieve the name of the resource with opposite HADROper attr
# Updates  : Name of the peer resource

sub get_peer_res_name {
	my $self = shift;

	my $temp;
	if ($self->{'HADROper'} =~ /LINK/i) {
		$temp = "SWITCH";
	} else {
		$temp = "LINK";
	}
	open(PRES, "$ag_i18n_inc::HARES -display -type Db2HADR -attribute ArgListValues |");
	while(<PRES>) {
		next unless /\s+$self->{'Db2dbRes'}\s/;
		next unless /\s+$self->{'DBName'}\s+/;
		next unless /\s+$temp\s+/i;
		chomp;
		($self->{'peer'}, undef) = split();
		last;
	}
	while (<PRES>) {next;}	# empty pipe
	close PRES;
	return 1 unless (defined $self->{'peer'});
	return 0;
}

# Function : lock_create
# Desc     : Creates a lock file to mark the resource as online
# Input    : $ResID
# Returns  : 0 - success
#
sub lock_create {
	my $self = shift;
	my $resname = shift;

        my $lfile = "$lfpath" . "$resname" . "$lfext";
        open(LCK, ">$lfile") or return 1;
        close LCK;
        return 0;
}

# Function : lock_exists
# Desc     : Checks if the lock file for a resource exists
# Input    : $ResID
# Returns  : 0 - file does not exist
#          : 1 - file exists
#
sub lock_exists {
	my $self = shift;
	my $resname = shift;

        my $lfile = "$lfpath" . "$resname" . "$lfext";
        return 1 if -f $lfile;
        return 0;
}

# Function : lock_remove
# Desc     : Removes the lock file for a resource to mark it offline
# Input    : $ResID
# Returns  : 0 - success
#
sub lock_remove {
	my $self = shift;
	my $resname = shift;

        my $lfile = "$lfpath" . "$resname" . "$lfext";
        unlink("$lfile");
        return 0;
}

# Function : peer_window_supported
# Desc     : Check if peer window is supported or not
# Input    : 
# Returns  : 1 - peer window supported.
#          : 0 - peer window not suported.
#
sub peer_window_supported {
	my $self = shift;
	my $version = `$self->{'db2licm'}`;
	if ($?) {
		VCSAG_LOG_MSG("E", "Failed to get db2 version", 202);
                return 0;
        }
       
	VCSAG_LOGDBG_MSG("1", "DB2 Version :: $version, sync_mode :: $self->{'sync_mode'}", 0, "$version","$self->{'sync_mode'}");
	
	if (($version >= 9.5) && (defined $self->{'sync_mode'}) && ($self->{'sync_mode'} eq 'Sync' || $self->{'sync_mode'} eq 'Nearsync')) {
		return 1;
	} else {
		return 0;
	}
}

1;
